//*****************************************************************************************//
//                                    FrameBuffers                                         //
//*****************************************************************************************//

#ifndef frameBufferModule
#define frameBufferModule 

//*****************************************************************************************//
//                                    FrameBuffers                                         //
//*****************************************************************************************//

class FrameBuffer {
	private:
		GLuint colorTextureAttachmentHandle;
		GLuint depthTextureAttachmentHandle;
		GLuint depthBufferAttachmentHandle;
		GLuint frameBufferHandle;

		bool hasColorAttachment, hasDepthAttachment;
		bool isBroken; bool isActivated; bool isCubeMap;
		GLuint textureType;

	public:
		unsigned int width;	unsigned int height;

		FrameBuffer (long width, long height, bool isCubeMap = false);
		~FrameBuffer (){detach ();};

		void attachDepthOnly ();
		void attachColorOnly (bool use16Bits = false);
		void attachColorAndDepth ();
		void detach ();

		//To activate the texture
		void activateAsTexture ();
		void deactivateAsTexture ();

		//To activate the depth texture
		void activateAsDepthTexture ();
		void deactivateAsDepthTexture ();

		//To activate the frame buffer (in order to write)
		void activate (); //For non-cube maps.
		void activate (long faceIndex); //For cube maps.
		void deactivate ();

	private:
		void setFiltering (GLint filter, GLint clamping, bool mipmap);
		bool isComplete ();
	public:
		static void FrameBuffer::example ();
};

#endif //frameBufferModule 
