//*****************************************************************************************//
//                                      Shaders                                            //
//*****************************************************************************************//

#ifndef shaderModule
#define shaderModule 

//*****************************************************************************************//
//                                      Extras                                             //
//*****************************************************************************************//

bool isExtensionSupported (const char *extension);

//*****************************************************************************************//
//                                      Shaders                                            //
//*****************************************************************************************//

class Shader {
	public:
	
		Shader (const std::string &name) {this->name = name; privateProgramHandle = 0; isBroken = false;}
		~Shader () {unload ();};

		void activate ();
		void deactivate ();

		//To pass information to the shader if it's activated.
		void setUniform1f (const std::string &variable, float value);
		void setUniform3f (const std::string &variable, float value0, float value1, float value2);
		void setUniform4f (const std::string &variable, float value0, float value1, float value2, float value3);
		void setUniform1i (const std::string &variable, long value);
		void setUniformTexture (const std::string &variable, long textureUnit);
		void setUniformMatrix4fv (const std::string &variable, float *matrix);
		
		void load ();
		void unload ();

		inline GLhandleARB handle () {return privateProgramHandle;}	
	private:
		GLhandleARB privateProgramHandle;
		std::string name;
		bool isBroken;

		GLhandleARB privateLoadAndCompileShader (const char *fileName, GLenum type);
		GLhandleARB privateLinkShaders (GLhandleARB *shaderHandles, const long shaderHandlesSize);
	public:
		static void example (); //Go look at this example...
};

//*****************************************************************************************//
//                                    ShaderManager                                        //
//*****************************************************************************************//

declareDictionary (Shader);

class ShaderManager;
extern ShaderManager *shaderManager;

class ShaderManager {
	public:
		static void ShaderManager::setup () {shaderManager = new ShaderManager ();}
		static void ShaderManager::wrapup () {delete shaderManager;}

		Shader *shaderNamed (const std::string &name);
		inline bool isSupported (const std::string &extension) {return isExtensionSupported ((const char *) extension.c_str ());};

	protected:
		ShaderDictionary shaders;

		ShaderManager () {}
		~ShaderManager () {deleteShaderDictionaryEntries (shaders);};
};

#endif //shaderModule 

