
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.all"

//*****************************************************************************************//
//                                      Utilities                                          //
//*****************************************************************************************//

void convertToLowercase (char *name) {
	for (long index = strlen (name) - 1; index >= 0; index--) {
		name [index] = tolower (name [index]);
	}
}

bool isExtensionSupported (const char *allExtensions, const char *extension) {
	//Extension names should not have spaces (so ignore it if it has a space by returning false).
	char *where = strchr (extension, ' ');
	if (where != NULL || *extension == '\0') return false;

	//It takes a bit of care to be fool-proof about parsing the OpenGL extensions string. 
	//Don't be fooled by sub-strings, etc
	const char *start = allExtensions;
	for (;;) {
		where = strstr ((const char *) start, extension);
		if (where == NULL) return false;
		char *terminator = where + strlen (extension);
		if (where == start || *(where - 1) == ' ') {
			if (*terminator == ' ' || *terminator == '\0') {return true;}
		}
		start = terminator;
	}
}

void logExtensions (bool logAllExtensions) {
	char *extensions = (char *) glGetString (GL_EXTENSIONS);

	//Log 3 specific extensions by specifically checking for them; one needed for face groups (the vertex buffer
	//extension), one needed for shaders (the shader extension), and one needed for frame buffers.
	
	bool isARBVertexBufferObjectExtensionPresent = isExtensionSupported (extensions, "GL_ARB_vertex_buffer_object");
	::log ("\nARB vertex buffer object extension %s supported", isARBVertexBufferObjectExtensionPresent ? "IS" : "IS NOT");

	bool isARBShaderExtensionPresent = isExtensionSupported (extensions, "GL_ARB_shading_language_100");
	::log ("\nARB shader extension %s supported", isARBShaderExtensionPresent ? "IS" : "IS NOT");

	bool isARBFrameBufferExtensionPresent = isExtensionSupported (extensions, "GL_ARB_framebuffer_object");
	::log ("\nARB frame buffer extension %s supported", isARBFrameBufferExtensionPresent ? "IS" : "IS NOT");

	if (!logAllExtensions) return;

	::log ("\nAvailable OpenGL extensions:");
	char buffer [256]; bool looping = true; char *next = extensions;
	while (looping) {
		char *end = strchr (next, ' '); if (end == NULL) end = strchr (next, '\0');
		long size = end - next;
		strncpy (buffer, next, size); buffer [size] = '\0';
		::log ("\n\t%s", buffer);
		next = end + 1;
		looping = end != NULL && *end != '\0';
	}
	::log ("\nDONE");
}


//*****************************************************************************************//
//                                Making Broken OpenGL Work                                //
//*****************************************************************************************//

//ARB vertex buffer objects
PFNGLBINDBUFFERARBPROC glBindBufferARB = NULL;
PFNGLDELETEBUFFERSARBPROC glDeleteBuffersARB = NULL;
PFNGLGENBUFFERSARBPROC glGenBuffersARB = NULL;
PFNGLISBUFFERARBPROC glIsBufferARB = NULL;
PFNGLBUFFERDATAARBPROC glBufferDataARB = NULL;
PFNGLBUFFERSUBDATAARBPROC glBufferSubDataARB = NULL;
PFNGLGETBUFFERSUBDATAARBPROC glGetBufferSubDataARB = NULL;
PFNGLMAPBUFFERARBPROC glMapBufferARB = NULL;
PFNGLUNMAPBUFFERARBPROC glUnmapBufferARB = NULL;
PFNGLGETBUFFERPARAMETERIVARBPROC glGetBufferParameterivARB = NULL;
PFNGLGETBUFFERPOINTERVARBPROC glGetBufferPointervARB = NULL;

PFNGLATTACHOBJECTARBPROC glAttachObjectARB = NULL;
PFNGLCOMPILESHADERARBPROC glCompileShaderARB = NULL;
PFNGLCREATEPROGRAMOBJECTARBPROC glCreateProgramObjectARB = NULL;
PFNGLCREATESHADEROBJECTARBPROC glCreateShaderObjectARB = NULL;
PFNGLDELETEOBJECTARBPROC glDeleteObjectARB = NULL;
PFNGLDETACHOBJECTARBPROC glDetachObjectARB = NULL;
PFNGLGETACTIVEUNIFORMARBPROC glActiveUniformARB = NULL;
PFNGLGETATTACHEDOBJECTSARBPROC glGetAttachedObjectARB = NULL;
PFNGLGETHANDLEARBPROC glGetHandleARB = NULL;
PFNGLGETINFOLOGARBPROC glGetInfoLogARB = NULL;
PFNGLGETOBJECTPARAMETERFVARBPROC glGetObjectParameterfvARB = NULL;
PFNGLGETOBJECTPARAMETERIVARBPROC glGetObjectParameterivARB = NULL;
PFNGLGETSHADERSOURCEARBPROC glGetShaderSourceARB = NULL;
PFNGLGETUNIFORMLOCATIONARBPROC glGetUniformLocationARB = NULL;
PFNGLGETUNIFORMFVARBPROC glGetUniformfvARB = NULL;
PFNGLGETUNIFORMIVARBPROC glGetUniformivARB = NULL;
PFNGLLINKPROGRAMARBPROC glLinkProgramARB = NULL;
PFNGLSHADERSOURCEARBPROC glShaderSourceARB = NULL;
PFNGLUNIFORM1FARBPROC glUniform1fARB = NULL;
PFNGLUNIFORM1FVARBPROC glUniform1fvARB = NULL;
PFNGLUNIFORM1IARBPROC glUniform1iARB = NULL;
PFNGLUNIFORM1IVARBPROC glUniform1ivARB = NULL;
PFNGLUNIFORM2FARBPROC glUniform2fARB = NULL;
PFNGLUNIFORM2FVARBPROC glUniform2fvARB = NULL;
PFNGLUNIFORM2IARBPROC glUniform2iARB = NULL;
PFNGLUNIFORM2IVARBPROC glUniform2ivARB = NULL;
PFNGLUNIFORM3FARBPROC glUniform3fARB = NULL;
PFNGLUNIFORM3FVARBPROC glUniform3fvARB = NULL;
PFNGLUNIFORM3IARBPROC glUniform3iARB = NULL;
PFNGLUNIFORM3IVARBPROC glUniform3ivARB = NULL;
PFNGLUNIFORM4FARBPROC glUniform4fARB = NULL;
PFNGLUNIFORM4FVARBPROC glUniform4fvARB = NULL;
PFNGLUNIFORM4IARBPROC glUniform4iARB = NULL;
PFNGLUNIFORM4IVARBPROC glUniform4ivARB = NULL;
PFNGLUNIFORMMATRIX2FVARBPROC glUniformMatrix2ARB = NULL;
PFNGLUNIFORMMATRIX3FVARBPROC glUniformMatrix3ARB = NULL;
PFNGLUNIFORMMATRIX4FVARBPROC glUniformMatrix4ARB = NULL;
PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB = NULL;
PFNGLVALIDATEPROGRAMARBPROC glValidateProgramARB = NULL;

void setupOpenOLDStyleOpenGLExtensions () {
	#if (USING_NEW_OPENGL)
		return; //Since nothing needs to be done...
	#endif //USING_NEW_OPENGL
	static bool alreadyExecuted = false; if (alreadyExecuted) return; alreadyExecuted = true;

	glBindBufferARB = (PFNGLBINDBUFFERARBPROC) wglGetProcAddress ("glBindBufferARB");
	glDeleteBuffersARB = (PFNGLDELETEBUFFERSARBPROC) wglGetProcAddress ("glDeleteBuffersARB");
	glGenBuffersARB = (PFNGLGENBUFFERSARBPROC) wglGetProcAddress ("glGenBuffersARB");
	glIsBufferARB = (PFNGLISBUFFERARBPROC) wglGetProcAddress ("glIsBufferARB");
	glBufferDataARB = (PFNGLBUFFERDATAARBPROC) wglGetProcAddress ("glBufferDataARB");
	glBufferSubDataARB = (PFNGLBUFFERSUBDATAARBPROC) wglGetProcAddress ("glBufferSubDataARB");
	glGetBufferSubDataARB = (PFNGLGETBUFFERSUBDATAARBPROC) wglGetProcAddress ("glGetBufferSubDataARB");
	glMapBufferARB = (PFNGLMAPBUFFERARBPROC) wglGetProcAddress ("glMapBufferARB");
	glUnmapBufferARB = (PFNGLUNMAPBUFFERARBPROC) wglGetProcAddress ("glUnmapBufferARB");
	glGetBufferParameterivARB = (PFNGLGETBUFFERPARAMETERIVARBPROC) wglGetProcAddress ("glGetBufferParameterivARB");
	glGetBufferPointervARB = (PFNGLGETBUFFERPOINTERVARBPROC) wglGetProcAddress ("glGetBufferPointervARB");

	glAttachObjectARB = (PFNGLATTACHOBJECTARBPROC) wglGetProcAddress ("glAttachObjectARB");
	glCompileShaderARB = (PFNGLCOMPILESHADERARBPROC) wglGetProcAddress ("glCompileShaderARB");
	glCreateProgramObjectARB = (PFNGLCREATEPROGRAMOBJECTARBPROC) wglGetProcAddress ("glCreateProgramObjectARB");
	glCreateShaderObjectARB = (PFNGLCREATESHADEROBJECTARBPROC) wglGetProcAddress ("glCreateShaderObjectARB");
	glDeleteObjectARB = (PFNGLDELETEOBJECTARBPROC) wglGetProcAddress ("glDeleteObjectARB");
	glDetachObjectARB = (PFNGLDETACHOBJECTARBPROC) wglGetProcAddress ("glDetachObjectARB");
	glActiveUniformARB = (PFNGLGETACTIVEUNIFORMARBPROC) wglGetProcAddress ("glActiveUniformARB");
	glGetAttachedObjectARB = (PFNGLGETATTACHEDOBJECTSARBPROC) wglGetProcAddress ("glGetAttachedObjectARB");
	glGetHandleARB = (PFNGLGETHANDLEARBPROC) wglGetProcAddress ("glGetHandleARB");
	glGetInfoLogARB = (PFNGLGETINFOLOGARBPROC) wglGetProcAddress ("glGetInfoLogARB");
	glGetObjectParameterfvARB = (PFNGLGETOBJECTPARAMETERFVARBPROC) wglGetProcAddress ("glGetObjectParameterfvARB");
	glGetObjectParameterivARB = (PFNGLGETOBJECTPARAMETERIVARBPROC) wglGetProcAddress ("glGetObjectParameterivARB");
	glGetShaderSourceARB = (PFNGLGETSHADERSOURCEARBPROC) wglGetProcAddress ("glGetShaderSourceARB");
	glGetUniformLocationARB = (PFNGLGETUNIFORMLOCATIONARBPROC) wglGetProcAddress ("glGetUniformLocationARB");
	glGetUniformfvARB = (PFNGLGETUNIFORMFVARBPROC) wglGetProcAddress ("glGetUniformfvARB");
	glGetUniformivARB = (PFNGLGETUNIFORMIVARBPROC) wglGetProcAddress ("glGetUniformivARB");
	glLinkProgramARB = (PFNGLLINKPROGRAMARBPROC) wglGetProcAddress ("glLinkProgramARB");
	glShaderSourceARB = (PFNGLSHADERSOURCEARBPROC) wglGetProcAddress ("glShaderSourceARB");
	glUniform1fARB = (PFNGLUNIFORM1FARBPROC) wglGetProcAddress ("glUniform1fARB");
	glUniform1fvARB = (PFNGLUNIFORM1FVARBPROC) wglGetProcAddress ("glUniform1fvARB");
	glUniform1iARB = (PFNGLUNIFORM1IARBPROC) wglGetProcAddress ("glUniform1iARB");
	glUniform1ivARB = (PFNGLUNIFORM1IVARBPROC) wglGetProcAddress ("glUniform1ivARB");
	glUniform2fARB = (PFNGLUNIFORM2FARBPROC) wglGetProcAddress ("glUniform2fARB");
	glUniform2fvARB = (PFNGLUNIFORM2FVARBPROC) wglGetProcAddress ("glUniform2fvARB");
	glUniform2iARB = (PFNGLUNIFORM2IARBPROC) wglGetProcAddress ("glUniform2iARB");
	glUniform2ivARB = (PFNGLUNIFORM2IVARBPROC) wglGetProcAddress ("glUniform2ivARB");
	glUniform3fARB = (PFNGLUNIFORM3FARBPROC) wglGetProcAddress ("glUniform3fARB");
	glUniform3fvARB = (PFNGLUNIFORM3FVARBPROC) wglGetProcAddress ("glUniform3fvARB");
	glUniform3iARB = (PFNGLUNIFORM3IARBPROC) wglGetProcAddress ("glUniform3iARB");
	glUniform3ivARB = (PFNGLUNIFORM3IVARBPROC) wglGetProcAddress ("glUniform3ivARB");
	glUniform4fARB = (PFNGLUNIFORM4FARBPROC) wglGetProcAddress ("glUniform4fARB");
	glUniform4fvARB = (PFNGLUNIFORM4FVARBPROC) wglGetProcAddress ("glUniform4fvARB");
	glUniform4iARB = (PFNGLUNIFORM4IARBPROC) wglGetProcAddress ("glUniform4iARB");
	glUniform4ivARB = (PFNGLUNIFORM4IVARBPROC) wglGetProcAddress ("glUniform4ivARB");
	glUniformMatrix2ARB = (PFNGLUNIFORMMATRIX2FVARBPROC) wglGetProcAddress ("glUniformMatrix2ARB");
	glUniformMatrix3ARB = (PFNGLUNIFORMMATRIX3FVARBPROC) wglGetProcAddress ("glUniformMatrix3ARB");
	glUniformMatrix4ARB = (PFNGLUNIFORMMATRIX4FVARBPROC) wglGetProcAddress ("glUniformMatrix4ARB");
	glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC) wglGetProcAddress ("glUseProgramObjectARB");
	glValidateProgramARB = (PFNGLVALIDATEPROGRAMARBPROC) wglGetProcAddress ("glValidateProgramARB");
}
