
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.all"

//*****************************************************************************************//
//                                       World                                             //
//*****************************************************************************************//

void World::setup () {
	//Nothing to do so far.
}

void World::wrapup () {
}

void World::tick () {
	//Tick all the objects in the world such as doors, sun, weather...
}

void World::draw () {
	//Draw all the objects in the world such as terrain, buildings, etc.
}

void World::log () {
	//For debugging, it might be useful to output the entire world into the log file.

	//The simple stuff first.
	::log ("\n\nWorld...\nStart position [%3.2f,%3.2f,%3.2f]", startPosition.x, startPosition.y, startPosition.z);

	//The objects: A version WITH loop macros.
	loopVector (objectIndex, objects)
		Object &object = *(objects [objectIndex]);
		object.log (objectIndex);
	endloop
	
	/*
	//The objects: A version WITHOUT loop macros.
	for (long objectIndex = 0; objectIndex < objects.size (); objectIndex++) {
		Object &object = *(objects [objectIndex]);
		object.log ();
	}
	*/
}

World *World::read () {
	//------------------------------------------------------------------------------------
	//FOR YOU TO EXTEND... Please look in builder since it can already import .uni files 
	//which are ALMOSTY the same...
	//------------------------------------------------------------------------------------

	//Prompt for a ".wrl" file (THIS IS YOUR OWN PERSONAL FORMAT) and reads it in to
	//a brand new world. You should also load all the textures you ultimately need
	//into the game card (see class Texture for how to do this to an individual texture).

	//Note: many faces will have the same texture name... Don't read in the texture bits
	//multiple times. The best way to do this is have collected the texture names together 
	//in one collection and output them before any of the objects are output.
	//Then, when you read the names in here, you can create a corresponding texture object.
	//Subsequently, when you read a face in, you can either look up the texture object 
	//by name if a name was exported with the face or you can simply index the collection
	//if you exported an index instead of a name (much easier).
 
	char *filename = promptForWorldRead ();
	if (filename == NULL) return NULL;

	World *world = new World;
	//world->import (input); //This method does not exist yet... You need to add it...

	::ifstream input; input.open (filename); 
	if (input.bad ()) {prompt ("Unable to open input file \"%s\".", filename); return NULL;}

	//------------------------------------------------------------------------------------
	//Look at how the builder does this to get started... You are going to do something
	//similar... HOWEVER, UNLIKE THE BUILDER... You will DISCARD ALL PROPERTIES
	//keeping only the information (if any) that might be useful. This kind of 
	//information will become useful once you start creating specialized subclasses
	//of Object; e.g., a Vehicle. In those cases, if there were a useful property
	//associated with the vehicle, say for example, its color, then you would 
	//read in the color information and store it in the vehicle's color variable
	//as opposed to a generic property dictionary...

	//Why are we not creating property dictionaries again? BECAUSE THEY ARE TOO
	//BIG AND SLOW AND UNNECESSARY TO HAVE in a game that needs all the space
	//it can get and all the speed it can get...
	//----------------------------------------------------------------------------

	input.close ();
	return world;
}

void World::unload () {
	//Unload all of the textures prior to deleting the world... If you don't, you will
	//reach a point where you will run in slow motion as the card thrashes around trying
	//to allow you to run in the memory it has left... Rebooting will fix it...
}