#version 130

uniform sampler2D texture;

in vec3 lightDirectionInCameraSpace;
in vec3 normalInCameraSpace;

in vec2 textureCoordinate; 
in vec4 pixelColor;

out vec4 finalColor; //Or special built-in variable gl_FragColor 

void main () {
	vec4 textureColor = texture2D (texture, textureCoordinate);
	vec4 unlitColor = textureColor * pixelColor;

	vec3 normalVector = normalize (normalInCameraSpace);
	vec3 lightVector = lightDirectionInCameraSpace; //It's constant...
	vec3 eyeVector = vec3 (0.0, 0.0, 1.0);

	//Dot product gives 0 when 90 degrees, maximum when 0 degrees...
	float diffuse = dot (normalVector, lightVector);

	vec3 halfAngle = normalize (eyeVector + lightVector);
	float specular = max (dot (normalVector, halfAngle), 0.0);

	finalColor = 
		unlitColor * diffuse +
		unlitColor * (4.0 * pow (specular, 32.0));
}
