#version 130

uniform vec3 lightDirection;
uniform float time;

out vec3 lightDirectionInCameraSpace;
out vec3 normalInCameraSpace;

out vec2 textureCoordinate; 
out vec4 pixelColor;

void main () {
	//Compute a modified vertex before transforming it...
	float timeFrequency = 8.0; float spaceFrequency = 4.0;
	float amount = 0.5 * (0.75 + sin (time * timeFrequency + (gl_Vertex.x + gl_Vertex.z) * spaceFrequency));
	vec4 position = vec4 (gl_Vertex.xyz + gl_Normal * amount * 0.215, gl_Vertex.w);

	//Convert the normal to camera space and pass along 
	//with the camera light direction also in camera space.
	mat3 rotationOnly = mat3 (gl_ModelViewMatrix);
	lightDirectionInCameraSpace = normalize (lightDirection);
	normalInCameraSpace = normalize (rotationOnly * gl_Normal); //Should use inverse transpose...

	textureCoordinate = gl_MultiTexCoord0.xy; //From vec4 to vec2
	gl_Position = gl_ProjectionMatrix * (gl_ModelViewMatrix * position);
	pixelColor = gl_Color;
}


